--[[
BarKeep, a World of Warcraft addon to manage the Blizzard Bars.
Copyright (C) 2007 Platykurtic.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
]]

-- Global -> Local for speed
local _G = getfenv(0)
local math_floor = math.floor
local MainMenuBar, BonusActionBarFrame, VehicleMenuBar = MainMenuBar, BonusActionBarFrame, VehicleMenuBar
local MainMenuBarArtFrame = MainMenuBarArtFrame
local MultiBarBottomLeft, MultiBarBottomRight, MultiBarLeft, MultiBarRight = MultiBarBottomLeft, MultiBarBottomRight, MultiBarLeft, MultiBarRight
local ShapeshiftBarFrame, PetActionBarFrame = ShapeshiftBarFrame, PetActionBarFrame
local InCombatLockdown = InCombatLockdown
local HasAction = HasAction
local playerclass = UnitClass("player")

-- Fix for vehicle repositioning for the micromenu and the vehicle buttons
function VehicleMenuBar_MoveMicroButtons(...) end
function VehicleMenuBar_SetSkin(...) end
--function VehicleSeatIndicator_SetUpVehicle(...) end

BarKeep_OnLoad = function(self, ...)
	-- Hide Some MainMenuBar, ShapeShiftBar and PossessBar Stuff
	local StuffToHide = {
		"MainMenuExpBar", "MainMenuBarMaxLevelBar", "ExhaustionTick", "ReputationWatchBar",
		"MainMenuXPBarTexture0", "MainMenuXPBarTexture1", "MainMenuXPBarTexture2", "MainMenuXPBarTexture3",
		"ReputationWatchBarTexture0", "ReputationWatchBarTexture1", "ReputationWatchBarTexture2", "ReputationWatchBarTexture3",
		"ReputationXPBarTexture0", "ReputationXPBarTexture1", "ReputationXPBarTexture2", "ReputationXPBarTexture3", "MainMenuBarOverlayFrame",
		"MainMenuBarTexture0", "MainMenuBarTexture1", "MainMenuBarTexture2", "MainMenuBarTexture3",
		"MainMenuBarLeftEndCap", "MainMenuBarRightEndCap", "MainMenuBarPageNumber",
		"MainMenuMaxLevelBar0", "MainMenuMaxLevelBar1", "MainMenuMaxLevelBar2", "MainMenuMaxLevelBar3",
		"BonusActionBarTexture0", "BonusActionBarTexture1",
		"SlidingActionBarTexture0", "SlidingActionBarTexture1",
		"ShapeshiftBarLeft", "ShapeshiftBarRight", "ShapeshiftBarMiddle",
		"PossessBackground1", "PossessBackground2",		
		"ActionBarUpButton","ActionBarDownButton",
		}
	for index,value in pairs(StuffToHide) do
		_G[value]:SetAlpha(0)
		_G[value]:Hide()
	end

	-- hides main actionbar when bonusactionbar appears
	function BarKeep_HideMainAction()
		for i = 1, 12, 1 do 
			_G["ActionButton"..i]:SetAlpha(0)
			-- Ensure you can't see the HotKey text for buttons without actions
			if not HasAction(_G["BonusActionButton"..i].action) then _G["BonusActionButton"..i.."HotKey"]:SetAlpha(0) end
		end
	end
	hooksecurefunc("ShowBonusActionBar", BarKeep_HideMainAction)
	function BarKeep_ShowMainAction() 
		for i = 1, 12, 1 do 
			_G["ActionButton"..i]:SetAlpha(1)
			-- Ensure that the HotKey text fix above is corrected - probably not needed
			_G["BonusActionButton"..i.."HotKey"]:SetAlpha(1)
		end
		BarKeep_MainBarUpdate()		
	end
	hooksecurefunc("HideBonusActionBar", BarKeep_ShowMainAction)
	if BonusActionBarFrame:IsShown() then BarKeep_HideMainAction() end

	-- Skin the backdrop textures for the BonusActionBar so click-throughs to the main bar aren't possible
	for i = 1, 12, 1 do
		local button = _G["BonusActionButton"..i]
		button:SetBackdrop(nil)
		-- Ensure the BonusActionButton  always remains shown
		button:SetAttribute("showgrid", 99)
		button:Show()
	end

	-- Pet Action Bar Frame Fix
	PetActionBarFrame:SetAttribute("unit", "pet")
	RegisterUnitWatch(PetActionBarFrame)
	BarKeepPetFrame:SetAttribute("unit", "pet")
	RegisterUnitWatch(BarKeepPetFrame)

	-- Manage the needed Frame Positions in BarKeep
	UIPARENT_MANAGED_FRAME_POSITIONS["MultiBarRight"] = nil
	UIPARENT_MANAGED_FRAME_POSITIONS["MultiBarLeft"] = nil
	UIPARENT_MANAGED_FRAME_POSITIONS["MultiBarBottomLeft"] = nil
	UIPARENT_MANAGED_FRAME_POSITIONS["MultiBarBottomRight"] = nil
	UIPARENT_MANAGED_FRAME_POSITIONS["PETACTIONBAR_YPOS"] = nil
	UIPARENT_MANAGED_FRAME_POSITIONS["ShapeshiftBarFrame"] = nil
	UIPARENT_MANAGED_FRAME_POSITIONS["PossessBarFrame"] = nil
	
	-- Register the events
	BarKeep_Frame:RegisterEvent("PLAYER_ENTERING_WORLD")
	BarKeep_Frame:RegisterEvent("ACTIONBAR_SHOWGRID")
	BarKeep_Frame:RegisterEvent("ACTIONBAR_HIDEGRID")
	BarKeep_Frame:RegisterEvent("ACTIONBAR_SLOT_CHANGED")

	-- hook the Frame position function and MultiActioBar update function so that BarKeep updates after the Blizzard bars
	hooksecurefunc("UIParent_ManageFramePosition", BarKeep_MainBarUpdate)
	hooksecurefunc("MultiActionBar_Update", BarKeep_MainBarUpdate)
	
	-- Hide Some Vehicle Stuff
	local StuffToHide = {
		"VehicleMenuBarArtFrame", "VehicleSeatIndicator", "VehicleMenuBarPitchSlider", "VehicleMenuBarActionButtonFrame"} --"VehicleSeatIndicator",
	for index,value in pairs(StuffToHide) do
		_G[value]:SetAlpha(0)
		_G[value]:Hide()
	end
	StuffToHide = nil
	
	-- Vehicle Action Bar Frame Fix
	BarKeepVehicleBarFrame:SetAttribute("unit", "vehicle")
	RegisterUnitWatch(BarKeepVehicleBarFrame)

	-- Manage the Vehicle Bar events and secure hooking
	BarKeep_Frame:RegisterEvent("UNIT_ENTERED_VEHICLE")
	BarKeep_Frame:RegisterEvent("UNIT_EXITED_VEHICLE")

	function BarKeep_VehicleBarSetSkin()
		-- Set the VehicleBar button sizing and textures - as per the Blizzard code
		VehicleMenuBarLeaveButton:SetHeight(47) VehicleMenuBarLeaveButton:SetWidth(50)
		VehicleMenuBarLeaveButton:GetNormalTexture():SetTexture([[Interface\Vehicles\UI-Vehicles-Button-Exit-Up]])
		VehicleMenuBarLeaveButton:GetNormalTexture():SetTexCoord(0.140625, 0.859375, 0.140625, 0.859375)
		VehicleMenuBarLeaveButton:GetPushedTexture():SetTexture([[Interface\Vehicles\UI-Vehicles-Button-Exit-Down]])
		VehicleMenuBarLeaveButton:GetPushedTexture():SetTexCoord(0.140625, 0.859375, 0.140625, 0.859375)
	
		VehicleMenuBarPitchUpButton:SetHeight(36) VehicleMenuBarPitchUpButton:SetWidth(38)
		VehicleMenuBarPitchUpButton:GetNormalTexture():SetTexture([[Interface\Vehicles\UI-Vehicles-Button-Pitch-Up]])
		VehicleMenuBarPitchUpButton:GetNormalTexture():SetTexCoord(0.21875, 0.765625, 0.234375, 0.78125)
		VehicleMenuBarPitchUpButton:GetPushedTexture():SetTexture([[Interface\Vehicles\UI-Vehicles-Button-Pitch-Down]])
		VehicleMenuBarPitchUpButton:GetPushedTexture():SetTexCoord(0.21875, 0.765625, 0.234375, 0.78125)

		VehicleMenuBarPitchDownButton:SetHeight(36) VehicleMenuBarPitchDownButton:SetWidth(38)
		VehicleMenuBarPitchDownButton:GetNormalTexture():SetTexture([[Interface\Vehicles\UI-Vehicles-Button-PitchDown-Up]])
		VehicleMenuBarPitchDownButton:GetNormalTexture():SetTexCoord(0.21875, 0.765625, 0.234375, 0.78125)
		VehicleMenuBarPitchDownButton:GetPushedTexture():SetTexture([[Interface\Vehicles\UI-Vehicles-Button-PitchDown-Down]])
		VehicleMenuBarPitchDownButton:GetPushedTexture():SetTexCoord(0.21875, 0.765625, 0.234375, 0.78125)
	end
	hooksecurefunc("VehicleMenuBar_ReleaseSkins", BarKeep_VehicleBarSetSkin) 
	BarKeep_VehicleBarSetSkin()
	
	BarKeep_ShowGrid = false
end

BarKeep_OnEvent = function(self, event, ...)
	local arg1, arg2, arg3, arg4, arg5 = ...
	if event == "PLAYER_ENTERING_WORLD" then
		BarKeep_MainBarUpdate()
		BarKeep_VehicleBarUpdate()
	elseif event == "ACTIONBAR_SHOWGRID"  then
		BarKeep_ShowGrid = true
		BarKeep_MainBarUpdate()
	elseif event == "ACTIONBAR_HIDEGRID" then
		BarKeep_ShowGrid = false
		BarKeep_MainBarUpdate()
	elseif event == "ACTIONBAR_SLOT_CHANGED" then
		-- Correct for the hotkey  alpha being set to 0 if there are changes to the BonusActionBars
		if arg1 >= 72 then
			for i = 1, 12, 1 do 
				if HasAction(_G["BonusActionButton"..i].action) then _G["BonusActionButton"..i.."HotKey"]:SetAlpha(1) else _G["BonusActionButton"..i.."HotKey"]:SetAlpha(0)	end
			end
		end
	elseif ((event == "UNIT_ENTERED_VEHICLE") and (arg1=="player")) then	
		if arg2 and (MainMenuBar.state == "vehicle") then -- Check that the vehicle interface will be used ... UnitHasVehicleUI("player") ) then
			BarKeepPetFrame:SetAlpha(0) PetActionBarFrame:SetAlpha(0)
			MultiBarBottomLeft:SetAlpha(0)
			MultiBarBottomRight:SetAlpha(0)
			MultiBarLeft:SetAlpha(0)
			MultiBarRight:SetAlpha(0)
--			if not InCombatLockdown() then BarKeep_VehicleBarUpdate() end
		end
	elseif ((event == "UNIT_EXITED_VEHICLE") and (arg1 == "player")) then
		if ( MainMenuBar.state == "player" ) then
			BarKeepPetFrame:SetAlpha(1)  PetActionBarFrame:SetAlpha(1)
			MultiBarBottomLeft:SetAlpha(1)
			MultiBarBottomRight:SetAlpha(1)
			MultiBarLeft:SetAlpha(1)
			MultiBarRight:SetAlpha(1)
--			if not InCombatLockdown() then BarKeep_MainBarUpdate() end
		end
	end
end

BarKeep_MainBarUpdate = function(...)
	BarKeep_ShowGrid = BarKeep_ShowGrid or ALWAYS_SHOW_MULTIBARS == "1" or ALWAYS_SHOW_MULTIBARS == 1
	
	-- Main Bars, BonusActionButtons
	if InCombatLockdown() then return end

	MainMenuBar:ClearAllPoints()
	MainMenuBar:SetAllPoints("BarKeepMainMenuBarFrame")
	MainMenuBar:SetFrameStrata("LOW")
	MainMenuExpBar:Hide()
	
	BonusActionBarFrame:ClearAllPoints()
	BonusActionBarFrame:SetAllPoints("MainMenuBar")
	BonusActionBarFrame:SetFrameStrata("MEDIUM")	
	
	local MainNumberButtons = BK_Setting.MainBarButtons
	local MainNumberRows = min(BK_Setting.MainBarRows, BK_Setting.MainBarButtons)
	local MainButtonPerRow = math_floor(MainNumberButtons/MainNumberRows + 0.5) -- just a precaution
	ActionButton1:ClearAllPoints()
	ActionButton1:SetPoint("TOPLEFT", "BarKeepMainMenuBarFrame", "TOPLEFT",9,4)
	ActionButton1:SetFrameStrata("LOW")

	BonusActionButton1:ClearAllPoints()
	BonusActionButton1:SetPoint("TOPLEFT", "BarKeepMainMenuBarFrame", "TOPLEFT",9,4)
	BonusActionButton1:SetFrameStrata("MEDIUM")
	for i = 2,12,1 do
		-- jump into a new row
		if ((i-1) % MainButtonPerRow) == 0 then
			_G["ActionButton"..i]:SetPoint("TOPLEFT", "ActionButton"..(i-MainButtonPerRow), "BOTTOMLEFT", 0, -3)
			_G["BonusActionButton"..i]:SetPoint("TOPLEFT", "BonusActionButton"..(i-MainButtonPerRow), "BOTTOMLEFT", 0, -3)
			-- align to the previous button
		else
			_G["ActionButton"..i]:SetPoint("TOPLEFT", "ActionButton"..(i-1), "TOPRIGHT", 3, 0)
			_G["BonusActionButton"..i]:SetPoint("TOPLEFT", "BonusActionButton"..(i-1), "TOPRIGHT", 3, 0)
		end
		_G["ActionButton"..i]:SetFrameStrata("LOW")
		_G["BonusActionButton"..i]:SetFrameStrata("MEDIUM")

		if i > MainNumberButtons then
			_G["ActionButton"..i]:SetAlpha(0)
			_G["ActionButton"..i]:Hide()
			_G["BonusActionButton"..i]:SetAlpha(0)
			_G["BonusActionButton"..i]:Hide()
		else
			_G["BonusActionButton"..i]:SetAlpha(1)
		end
	end
	
	-- Multi Bottom Bars
	if InCombatLockdown() then return end
	MultiBarBottomLeft:ClearAllPoints()
	MultiBarBottomLeft:SetFrameStrata("MEDIUM")
	MultiBarBottomLeft:SetAllPoints("BarKeepMultiBarBottomLeftFrame")
	MultiBarBottomLeftButton1:ClearAllPoints()
	MultiBarBottomLeftButton1:SetPoint("TOPLEFT", "BarKeepMultiBarBottomLeftFrame", "TOPLEFT", 0, 0)
	local LeftNumberButtons = BK_Setting.LeftBarButtons
	local LeftNumberRows = min(BK_Setting.LeftBarRows, BK_Setting.LeftBarButtons)
	local LeftButtonPerRow = math_floor(LeftNumberButtons/LeftNumberRows + 0.5) -- just a precaution
	for i = 2,12,1 do
		local button=_G["MultiBarBottomLeftButton"..i]
		-- jump into a new row
		if ((i-1) % LeftButtonPerRow) == 0 then
			button:SetPoint("TOPLEFT", "MultiBarBottomLeftButton"..(i-LeftButtonPerRow), "BOTTOMLEFT", 0, -3)
		-- align to the previous button
		else
			button:SetPoint("TOPLEFT", "MultiBarBottomLeftButton"..(i-1), "TOPRIGHT", 3, 0)
		end
		-- hide the unneeded buttons
		if i > LeftNumberButtons then
			button:SetAlpha(0)
			button:Hide()
		-- hack for hiding the grid if it appears
		elseif (not BarKeep_ShowGrid and not HasAction(button.action)) then
			button:Hide()
		else
			button:Show()
		end
	end
	
	if InCombatLockdown() then return end
	MultiBarBottomRight:ClearAllPoints()
	MultiBarBottomRight:SetFrameStrata("MEDIUM")
	MultiBarBottomRight:SetAllPoints("BarKeepMultiBarBottomRightFrame")
	MultiBarBottomRightButton1:ClearAllPoints()
	MultiBarBottomRightButton1:SetPoint("TOPLEFT", "BarKeepMultiBarBottomRightFrame", "TOPLEFT", 0, 0)
	local RightNumberButtons = BK_Setting.RightBarButtons
	local RightNumberRows = min(BK_Setting.RightBarRows, BK_Setting.RightBarButtons)
	local RightButtonPerRow = math_floor(RightNumberButtons/RightNumberRows + 0.5) -- just a precaution
	for i = 2,12,1 do
		local button=_G["MultiBarBottomRightButton"..i]
		-- jump into a new row
		if ((i-1) % RightButtonPerRow) == 0 then
			button:SetPoint("TOPLEFT", "MultiBarBottomRightButton"..(i-RightButtonPerRow), "BOTTOMLEFT", 0, -3)
		-- align to the previous button
		else
			button:SetPoint("TOPLEFT", "MultiBarBottomRightButton"..(i-1), "TOPRIGHT", 3, 0)
		end
		-- hide the unneeded buttons
		if i > RightNumberButtons then
			button:SetAlpha(0)
			button:Hide()
		-- hack for hiding the grid if it appears
		elseif (not BarKeep_ShowGrid and not HasAction(button.action)) then
			button:Hide()
		else
			button:Show()
		end
	end

	-- Shape/Stance/Form Bar
	if InCombatLockdown() then return end
	ShapeshiftButton1:ClearAllPoints()
	ShapeshiftButton1:SetParent("BarKeepShapeFrame")
	ShapeshiftButton1:SetPoint("TOPLEFT","BarKeepShapeFrame","TOPLEFT",0,0)
	ShapeshiftButton1:SetFrameStrata("MEDIUM")
	for i=2,10,1 do
		_G["ShapeshiftButton"..i]:SetParent("BarKeepShapeFrame")
		_G["ShapeshiftButton"..i]:SetPoint("TOPLEFT", "ShapeshiftButton"..(i-1), "TOPRIGHT", 3, 0)
		_G["ShapeshiftButton"..i]:SetFrameStrata("MEDIUM")
	end
	if ShapeshiftBarFrame:IsShown() and BK_Setting.ShapeBarHide then
		ShapeshiftBarFrame:Hide()
		BarKeepShapeFrame:Hide()
	end

	-- Totem Bar
	if InCombatLockdown() then return end
	if (playerclass == "SHAMAN" ) then
		TotemFrameTotem1:ClearAllPoints()
		TotemFrameTotem1:SetParent("BarKeepTotemFrame")
		TotemFrameTotem1:SetPoint("TOPLEFT","BarKeepTotemFrame","TOPLEFT",0,0)
		TotemFrameTotem1:SetFrameStrata("MEDIUM")
		for i=2,4,1 do
			_G["TotemFrameTotem"..i]:SetParent("BarKeepTotemFrame")
			_G["TotemFrameTotem"..i]:SetPoint("TOPLEFT", "TotemFrameTotem"..(i-1), "TOPRIGHT", -4, 0)
			_G["TotemFrameTotem"..i]:SetFrameStrata("MEDIUM")
		end
	end
	
	-- Pet Bar
	if InCombatLockdown() then return end
	PetActionBarFrame:ClearAllPoints()
	PetActionBarFrame:SetAllPoints("BarKeepPetFrame")
	PetActionButton1:ClearAllPoints()
	PetActionButton1:SetParent("BarKeepPetFrame")
	PetActionButton1:SetPoint("TOPLEFT","BarKeepPetFrame","TOPLEFT",0,0)
	PetActionButton1:SetFrameStrata("MEDIUM")
	local PetNumberRows = min(BK_Setting.PetBarRows, 10)
	local PetButtonPerRow = math_floor(10/PetNumberRows + 0.5) -- just a precaution
	for i = 2,10,1 do
		local button=_G["PetActionButton"..i]
		button:SetParent("BarKeepPetFrame")
		-- jump into a new row
		if ((i-1) % PetButtonPerRow) == 0 then
			button:SetPoint("TOPLEFT", "PetActionButton"..(i-PetButtonPerRow), "BOTTOMLEFT", 0, -3)
		-- align to the previous button
		else
			button:SetPoint("TOPLEFT", "PetActionButton"..i-1, "TOPRIGHT", 3, 0)
		end
		button:SetFrameStrata("MEDIUM")
	end

	-- Possession Bar
	if InCombatLockdown() then return end
	PossessBarFrame:ClearAllPoints()
	PossessBarFrame:SetFrameStrata("BACKGROUND")
	PossessBarFrame:SetPoint("TOPLEFT", "BarKeepPossessFrame", "TOPLEFT", 0, -5)
	PossessButton1:ClearAllPoints()
	PossessButton1:SetPoint("TOPLEFT", "PossessBarFrame", "TOPLEFT", 0, 0)
	PossessButton1:SetFrameStrata("MEDIUM")
	PossessButton2:ClearAllPoints()
	PossessButton2:SetPoint("TOPLEFT", "PossessButton1", "TOPRIGHT", 3, 0)
	PossessButton2:SetFrameStrata("MEDIUM")
	
	-- Multi Bars
	if InCombatLockdown() then return end
	MultiBarLeft:ClearAllPoints()
	MultiBarLeft:SetFrameStrata("MEDIUM")
	MultiBarLeft:SetAllPoints("BarKeepMultiBarLeftFrame")
	MultiBarLeftButton1:ClearAllPoints()
	MultiBarLeftButton1:SetPoint("TOPLEFT", "BarKeepMultiBarLeftFrame", "TOPLEFT", 0, 0)
	local MultiLeftNumberButtons = BK_Setting.MultiBarLeftButtons
	local MultiLeftNumberRows = min(BK_Setting.MultiBarLeftRows, BK_Setting.MultiBarLeftButtons)
	local MultiLeftButtonPerRow = math_floor(MultiLeftNumberButtons/MultiLeftNumberRows + 0.5) -- just a precaution	
	for i = 2,12,1 do
		local button=_G["MultiBarLeftButton"..i]
		-- jump into a new row
		if ((i-1) % MultiLeftButtonPerRow) == 0 then
			_G["MultiBarLeftButton"..i]:SetPoint("TOPLEFT", "MultiBarLeftButton"..(i-MultiLeftButtonPerRow), "BOTTOMLEFT", 0, -3)
		-- align to the previous button
		else
			_G["MultiBarLeftButton"..i]:SetPoint("TOPLEFT", "MultiBarLeftButton"..(i-1), "TOPRIGHT", 3, 0)
		end
		-- hide the unneeded buttons
		if i > MultiLeftNumberButtons then
			button:SetAlpha(0)
			button:Hide()
		-- hack for hiding the grid if it appears
		elseif (not BarKeep_ShowGrid and not HasAction(button.action)) then
			button:Hide()
		else
			button:Show()
		end
	end

	if InCombatLockdown() then return end	
	MultiBarRight:ClearAllPoints()
	MultiBarRight:SetFrameStrata("MEDIUM")
	MultiBarRight:SetAllPoints("BarKeepMultiBarRightFrame")
	MultiBarRightButton1:ClearAllPoints()
	MultiBarRightButton1:SetPoint("TOPLEFT", "BarKeepMultiBarRightFrame", "TOPLEFT", 0, 0)
	local MultiRightNumberButtons = BK_Setting.MultiBarRightButtons
	local MultiRightNumberRows = min(BK_Setting.MultiBarRightRows, BK_Setting.MultiBarRightButtons)
	local MultiRightButtonPerRow = math_floor(MultiRightNumberButtons/MultiRightNumberRows + 0.5) -- just a precaution
	for i = 2,12,1 do
		local button=_G["MultiBarRightButton"..i]
		-- jump into a new row
		if ((i-1) % MultiRightButtonPerRow) == 0 then
			_G["MultiBarRightButton"..i]:SetPoint("TOPLEFT", "MultiBarRightButton"..(i-MultiRightButtonPerRow), "BOTTOMLEFT", 0, -3)
		-- align to the previous button
		else
			_G["MultiBarRightButton"..i]:SetPoint("TOPLEFT", "MultiBarRightButton"..(i-1), "TOPRIGHT", 3, 0)
		end
		-- hide the unneeded buttons
		if i > MultiRightNumberButtons then
			button:SetAlpha(0)
			button:Hide()
		-- hack for hiding the grid if it appears
		elseif (not BarKeep_ShowGrid and not HasAction(button.action)) then
			button:Hide()
		else
			button:Show()
		end
	end
	
	-- MicroMenu and Bag Bars
	if InCombatLockdown() then return end
	local MainMenuStuff = {
		"CharacterMicroButton", "SpellbookMicroButton", "TalentMicroButton", "AchievementMicroButton", "QuestLogMicroButton",
		"SocialsMicroButton", "PVPMicroButton", "LFGMicroButton", "MainMenuMicroButton", "HelpMicroButton",
		}
	for index,value in pairs(MainMenuStuff) do
		_G[value]:ClearAllPoints()
		_G[value]:SetParent("BarKeepMainMenuFrame")
		_G[value]:SetFrameStrata("MEDIUM")
		_G[value]:SetScale(0.8)
	end
	CharacterMicroButton:SetPoint("TOPLEFT","BarKeepMainMenuFrame","TOPLEFT",0,0)
	SpellbookMicroButton:SetPoint("BOTTOMLEFT","CharacterMicroButton","BOTTOMRIGHT",-3,0)
	TalentMicroButton:SetPoint("BOTTOMLEFT","SpellbookMicroButton","BOTTOMRIGHT",-3,0)
	UpdateTalentButton()				-- Use the Blizzard code to place the Achievement button to avoid a potential LUA error
	AchievementMicroButton_Update()		-- Use the Blizzard code to place the QuestLog button avoid a potential LUA error
	SocialsMicroButton:SetPoint("BOTTOMLEFT","QuestLogMicroButton","BOTTOMRIGHT",-3,0)
	PVPMicroButton:SetPoint("BOTTOMLEFT","SocialsMicroButton","BOTTOMRIGHT",-3,0)
	LFGMicroButton:SetPoint("BOTTOMLEFT","PVPMicroButton","BOTTOMRIGHT",-3,0)
	MainMenuMicroButton:SetPoint("BOTTOMLEFT","LFGMicroButton","BOTTOMRIGHT",-3,0)	
	HelpMicroButton:SetPoint("BOTTOMLEFT","MainMenuMicroButton","BOTTOMRIGHT",-3,0)
	
	
	if InCombatLockdown() then return end
	local BagStuff = {
		"MainMenuBarBackpackButton", --"CharacterBag0Slot", "CharacterBag1Slot", "CharacterBag2Slot",	"CharacterBag3Slot", "KeyRingButton",
	}
	for index,value in pairs(BagStuff) do
		_G[value]:ClearAllPoints()
		_G[value]:SetFrameStrata("MEDIUM")
	end
	MainMenuBarBackpackButton:SetPoint("TOPRIGHT","BarKeepBagFrame","TOPRIGHT",0,0)
end

BarKeep_VehicleBarUpdate = function(...)
	-- Vehicle Bar
	if InCombatLockdown() then return end
	VehicleMenuBarActionButtonFrame:ClearAllPoints()
	VehicleMenuBarActionButtonFrame:SetAllPoints("BarKeepVehicleBarFrame")
	VehicleMenuBarActionButtonFrame:SetFrameStrata("HIGH")

	if InCombatLockdown() then return end
	VehicleMenuBarActionButton1:SetParent("BarKeepVehicleBarFrame")
	VehicleMenuBarActionButton1:ClearAllPoints()
	VehicleMenuBarActionButton1:SetPoint("TOPLEFT", "BarKeepVehicleBarFrame", "TOPLEFT", 0, 0)
	VehicleMenuBarActionButton1:SetFrameStrata("HIGH")
	for i = 2,6,1 do
		_G["VehicleMenuBarActionButton"..i]:SetParent("BarKeepVehicleBarFrame")
		_G["VehicleMenuBarActionButton"..i]:ClearAllPoints()
		_G["VehicleMenuBarActionButton"..i]:SetPoint("TOPLEFT", "VehicleMenuBarActionButton"..(i-1), "TOPRIGHT", 3, 0)
		_G["VehicleMenuBarActionButton"..i]:SetFrameStrata("HIGH")
	end

	if InCombatLockdown() then return end

	VehicleMenuBarPitchUpButton:SetParent("BarKeepVehicleBarFrame")
	VehicleMenuBarPitchUpButton:ClearAllPoints()
	VehicleMenuBarPitchUpButton:SetPoint("BOTTOMLEFT", "VehicleMenuBarActionButton6", "RIGHT", 3, 2)
	VehicleMenuBarPitchUpButton:SetFrameStrata("HIGH")

	VehicleMenuBarPitchDownButton:SetParent("BarKeepVehicleBarFrame")
	VehicleMenuBarPitchDownButton:ClearAllPoints()
	VehicleMenuBarPitchDownButton:SetPoint("TOPLEFT", "VehicleMenuBarActionButton6", "RIGHT", 3, -2)
	VehicleMenuBarPitchDownButton:SetFrameStrata("HIGH")

	VehicleMenuBarLeaveButton:SetParent("BarKeepVehicleBarFrame")
	VehicleMenuBarLeaveButton:ClearAllPoints()
	VehicleMenuBarLeaveButton:SetPoint("LEFT", "VehicleMenuBarPitchUpButton", "BOTTOMRIGHT", 3, -2)
	VehicleMenuBarLeaveButton:SetFrameStrata("HIGH")

--[[	VehicleSeatIndicator:SetParent("BarKeepVehicleBarFrame")
	VehicleSeatIndicator:ClearAllPoints()
	VehicleSeatIndicator:SetPoint("LEFT", "VehicleMenuBarLeaveButton", "RIGHT", 3, 0)
	VehicleSeatIndicator:SetFrameStrata("HIGH")]]
	
	-- MicroMenu Bar
	if InCombatLockdown() then return end
	BarKeepMainMenuFrame:SetFrameStrata("BACKGROUND")
	local MainMenuStuff = {
		"CharacterMicroButton", "SpellbookMicroButton", "TalentMicroButton", "AchievementMicroButton", "QuestLogMicroButton",
		"SocialsMicroButton", "PVPMicroButton", "LFGMicroButton", "MainMenuMicroButton", "HelpMicroButton",
		}
	for index,value in pairs(MainMenuStuff) do
		_G[value]:ClearAllPoints()
		_G[value]:SetParent("BarKeepMainMenuFrame")
		_G[value]:SetFrameStrata("MEDIUM")
		_G[value]:SetScale(0.8)
		if not BK_Setting.MenuBarHide then _G[value]:Show() end
	end
	CharacterMicroButton:SetPoint("TOPLEFT","BarKeepMainMenuFrame","TOPLEFT",0,0)
	SpellbookMicroButton:SetPoint("BOTTOMLEFT","CharacterMicroButton","BOTTOMRIGHT",-3,0)
	TalentMicroButton:SetPoint("BOTTOMLEFT","SpellbookMicroButton","BOTTOMRIGHT",-3,0)
	UpdateTalentButton()				-- Use the Blizzard code to place the Achievement button to avoid a potential LUA error
	AchievementMicroButton_Update()		-- Use the Blizzard code to place the QuestLog button avoid a potential LUA error
	SocialsMicroButton:SetPoint("BOTTOMLEFT","QuestLogMicroButton","BOTTOMRIGHT",-3,0)
	PVPMicroButton:SetPoint("BOTTOMLEFT","SocialsMicroButton","BOTTOMRIGHT",-3,0)
	LFGMicroButton:SetPoint("BOTTOMLEFT","PVPMicroButton","BOTTOMRIGHT",-3,0)
	MainMenuMicroButton:SetPoint("BOTTOMLEFT","LFGMicroButton","BOTTOMRIGHT",-3,0)	
	HelpMicroButton:SetPoint("BOTTOMLEFT","MainMenuMicroButton","BOTTOMRIGHT",-3,0)
end
